]> sigrok.org Git - libsigrok.git/blobdiff - filter.c
A few more random 'probe' to 'channel' changes.
[libsigrok.git] / filter.c
index 5698d333f0eea57de70aa97c5ca5277a1c2f4312..8e74b10d0f0f4ae8742b1ea3bba77243eaefc924 100644 (file)
--- a/filter.c
+++ b/filter.c
@@ -1,7 +1,7 @@
 /*
- * This file is part of the sigrok project.
+ * This file is part of the libsigrok project.
  *
- * Copyright (C) 2010 Bert Vermeulen <bert@biot.com>
+ * Copyright (C) 2010-2012 Bert Vermeulen <bert@biot.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
-#include <sigrok.h>
-#include "sigrok-internal.h"
+#include <glib.h>
+#include "libsigrok.h"
+#include "libsigrok-internal.h"
+
+#define LOG_PREFIX "filter"
+
+/**
+ * @file
+ *
+ * Helper functions to filter out unused channels from samples.
+ */
+
+/**
+ * @defgroup grp_filter Channel filter
+ *
+ * Helper functions to filter out unused channels from samples.
+ *
+ * @{
+ */
 
 /**
* Remove unused probes from samples.
  *
* Convert sample from maximum probes -- the way the hardware driver sent
  * it -- to a sample taking up only as much space as required, with
* unused probes removed.
  *
* The "unit size" is the number of bytes used to store probe values.
  * For example, a unit size of 1 means one byte is used (which can store
* 8 probe values, each of them is 1 bit). A unit size of 2 means we can
* store 16 probe values, 3 means we can store 24 probe values, and so on.
  *
  * If the data coming from the logic analyzer has a unit size of 4 for
* example (as the device has 32 probes), but only 2 of them are actually
  * used in an acquisition, this function can convert the samples to only
  * use up 1 byte per sample (unit size = 1) instead of 4 bytes per sample.
  *
* The output will contain the probe values in the order specified via the
* probelist. For example, if in_unitsize = 4, probelist = [5, 16, 30], and
  * out_unitsize = 1, then the output samples (each of them one byte in size)
* will have the following format: bit 0 = value of probe 5, bit 1 = value
* of probe 16, bit 2 = value of probe 30. Unused bit(s) in the output byte(s)
  * are zero.
  *
  * The caller must make sure that length_in is not bigger than the memory
  *
  * @param in_unitsize The unit size (>= 1) of the input (data_in).
  * @param out_unitsize The unit size (>= 1) the output shall have (data_out).
- * @param probelist Pointer to a list of integers (probe numbers). The probe
- *                  numbers in this list are 1-based, i.e. the first probe
- *                  is expected to be numbered 1 (not 0!). Must not be NULL.
+ *                     The requested unit size must be big enough to hold as
+ *                     much data as is specified by the number of enabled
+ *                     channels in 'channellist'.
+ * @param channel_array Pointer to a list of channel numbers, numbered starting
+ *                    from 0. The list is terminated with -1.
  * @param data_in Pointer to the input data buffer. Must not be NULL.
  * @param length_in The input data length (>= 1), in number of bytes.
  * @param data_out Variable which will point to the newly allocated buffer
  *         or SR_ERR_ARG upon invalid arguments.
  *         If something other than SR_OK is returned, the values of
  *         out_unitsize, data_out, and length_out are undefined.
+ *
+ * @since 0.2.0
  */
-int sr_filter_probes(int in_unitsize, int out_unitsize, const int *probelist,
-                    const unsigned char *data_in, uint64_t length_in,
-                    char **data_out, uint64_t *length_out)
+SR_API int sr_filter_channels(unsigned int in_unitsize, unsigned int out_unitsize,
+                           const GArray *channel_array, const uint8_t *data_in,
+                           uint64_t length_in, uint8_t **data_out,
+                           uint64_t *length_out)
 {
        unsigned int in_offset, out_offset;
-       int num_enabled_probes, out_bit, i;
-       uint64_t sample_in, sample_out;
+       int *channellist, out_bit;
+       unsigned int i;
+       uint8_t *sample_in, *sample_out;
 
-       if (!probelist) {
-               sr_err("filter: %s: probelist was NULL", __func__);
+       if (!channel_array) {
+               sr_err("%s: channel_array was NULL", __func__);
                return SR_ERR_ARG;
        }
+       channellist = (int *)channel_array->data;
 
        if (!data_in) {
-               sr_err("filter: %s: data_in was NULL", __func__);
+               sr_err("%s: data_in was NULL", __func__);
                return SR_ERR_ARG;
        }
 
        if (!data_out) {
-               sr_err("filter: %s: data_out was NULL", __func__);
+               sr_err("%s: data_out was NULL", __func__);
                return SR_ERR_ARG;
        }
 
        if (!length_out) {
-               sr_err("filter: %s: length_out was NULL", __func__);
+               sr_err("%s: length_out was NULL", __func__);
+               return SR_ERR_ARG;
+       }
+
+       /* Are there more channels than the target unit size supports? */
+       if (channel_array->len > out_unitsize * 8) {
+               sr_err("%s: too many channels (%d) for the target unit "
+                      "size (%d)", __func__, channel_array->len, out_unitsize);
                return SR_ERR_ARG;
        }
 
        if (!(*data_out = g_try_malloc(length_in))) {
-               sr_err("filter: %s: data_out malloc failed", __func__);
+               sr_err("%s: data_out malloc failed", __func__);
                return SR_ERR_MALLOC;
        }
 
-       num_enabled_probes = 0;
-       for (i = 0; probelist[i]; i++)
-               num_enabled_probes++;
-
-       if (num_enabled_probes == in_unitsize * 8) {
-               /* All probes are used -- no need to compress anything. */
+       if (channel_array->len == in_unitsize * 8) {
+               /* All channels are used -- no need to compress anything. */
                memcpy(*data_out, data_in, length_in);
                *length_out = length_in;
                return SR_OK;
        }
 
/* If we reached this point, not all probes are used, so "compress". */
        in_offset = out_offset = 0;
        while (in_offset <= length_in - in_unitsize) {
-               memcpy(&sample_in, data_in + in_offset, in_unitsize);
-               sample_out = out_bit = 0;
-               for (i = 0; probelist[i]; i++) {
-                       if (sample_in & (1 << (probelist[i] - 1)))
-                               sample_out |= (1 << out_bit);
+               sample_in = (uint8_t *)data_in + in_offset;
+               sample_out = (*data_out) + out_offset;
+               memset(sample_out, 0, out_unitsize);
+               out_bit = 0;
+               for (i = 0; i < channel_array->len; i++) {
+                       if (sample_in[channellist[i]>>3] & (1 << (channellist[i]&7)))
+                               sample_out[out_bit>>3] |= (1 << (out_bit&7));
                        out_bit++;
                }
-               memcpy((*data_out) + out_offset, &sample_out, out_unitsize);
                in_offset += in_unitsize;
                out_offset += out_unitsize;
        }
@@ -132,3 +160,5 @@ int sr_filter_probes(int in_unitsize, int out_unitsize, const int *probelist,
 
        return SR_OK;
 }
+
+/** @} */
@@ -1,7 +1,7 @@
 
 /**
- * Remove unused probes from samples.
+ * Remove unused channels from samples.
  *
@@ -1,7 +1,7 @@
  *
- * Convert sample from maximum probes -- the way the hardware driver sent
+ * Convert sample from maximum channels -- the way the hardware driver sent
  * it -- to a sample taking up only as much space as required, with
@@ -1,7 +1,7 @@
  * it -- to a sample taking up only as much space as required, with
- * unused probes removed.
+ * unused channels removed.
  *
@@ -1,7 +1,7 @@
  *
- * The "unit size" is the number of bytes used to store probe values.
+ * The "unit size" is the number of bytes used to store channel values.
  * For example, a unit size of 1 means one byte is used (which can store
@@ -1,7 +1,7 @@
  * For example, a unit size of 1 means one byte is used (which can store
- * 8 probe values, each of them is 1 bit). A unit size of 2 means we can
- * store 16 probe values, 3 means we can store 24 probe values, and so on.
+ * 8 channel values, each of them is 1 bit). A unit size of 2 means we can
+ * store 16 channel values, 3 means we can store 24 channel values, and so on.
  *
  * If the data coming from the logic analyzer has a unit size of 4 for
@@ -1,7 +1,7 @@
  *
  * If the data coming from the logic analyzer has a unit size of 4 for
- * example (as the device has 32 probes), but only 2 of them are actually
+ * example (as the device has 32 channels), but only 2 of them are actually
  * used in an acquisition, this function can convert the samples to only
  * use up 1 byte per sample (unit size = 1) instead of 4 bytes per sample.
@@ -1,7 +1,7 @@
  * use up 1 byte per sample (unit size = 1) instead of 4 bytes per sample.
  *
- * The output will contain the probe values in the order specified via the
- * probelist. For example, if in_unitsize = 4, probelist = [5, 16, 30], and
+ * The output will contain the channel values in the order specified via the
+ * channellist. For example, if in_unitsize = 4, channellist = [5, 16, 30], and
  * out_unitsize = 1, then the output samples (each of them one byte in size)
@@ -1,7 +1,7 @@
  * out_unitsize = 1, then the output samples (each of them one byte in size)
- * will have the following format: bit 0 = value of probe 5, bit 1 = value
- * of probe 16, bit 2 = value of probe 30. Unused bit(s) in the output byte(s)
+ * will have the following format: bit 0 = value of channel 5, bit 1 = value
+ * of channel 16, bit 2 = value of channel 30. Unused bit(s) in the output byte(s)
  * are zero.
  *
@@ -1,7 +1,7 @@
        }
 
-       /* If we reached this point, not all probes are used, so "compress". */
+       /* If we reached this point, not all channels are used, so "compress". */
        in_offset = out_offset = 0;
        while (in_offset <= length_in - in_unitsize) {